//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qwidget.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qfont.h>
#include <qpixmap.h>
#include <qpushbutton.h>

//Projectwide includes
#include "subalbumsWidget.h"
#include "subalbumPreviewWidget.h"
#include "subalbumsIconView.h"
#include "questionDialog.h"
#include "layoutWidget.h"
#include "window.h"
#include "titleWidget.h"
#include "../backend/album.h"
#include "../backend/subalbum.h"
#include "../config.h"

#include <iostream.h>

//==============================================
SubalbumsWidget::SubalbumsWidget(QWidget *parent, 
                                 const char* name ) : 
                                 QWidget(parent,name)
{
  //set layout pointer
  layout = (LayoutWidget*)parent;

  //create subalbums text
  subalbumsText = new QLabel( this );
  subalbumsText->setText( "Subalbums: " );
  subalbumsText->setFont( QFont( "Times", 12, QFont::Bold ) );

  //create subalbums iconview
  subalbums = new SubalbumsIconView( this );
  
  //establish a top-down view such that the scrollbar is always placed on the right
  subalbums->setArrangement( QIconView::LeftToRight );
  subalbums->setVScrollBarMode( QScrollView::AlwaysOn );

  //only allow one subalbum to be selected at a time
  subalbums->setSelectionMode( QIconView::Single ) ;

  //set auto-scroll off for drag-n-drop
  subalbums->setDragAutoScroll(true);
  subalbums->setAcceptDrops(true);

  //connect drop event on iconview to reorder slot
  connect( subalbums, SIGNAL(itemHasMoved()), SLOT(reorder()) );  

  //connect selectionChanged signal to updateSubalbumLayout method
  connect( subalbums, SIGNAL(selectionChanged(QIconViewItem*)),
           SLOT(updateSubalbumLayout(QIconViewItem*)) );

  //connect rightButtonClicked signal to updateSubalbumLayout method
  connect( subalbums, SIGNAL(rightButtonClicked(QIconViewItem*, const QPoint&)),
           SLOT(updateSubalbumLayout(QIconViewItem*)) );
           
  //create create/delete subalbum buttons
  createSubalbumImage = new QPixmap( QString(IMAGE_PATH)+"create.png" );
  
  createSubalbum = new QPushButton( this );
  createSubalbum->setPixmap( *createSubalbumImage );
  createSubalbum->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );  
  connect( createSubalbum, SIGNAL(clicked()), SLOT(createAction()) );

  deleteSubalbumImage = new QPixmap( QString(IMAGE_PATH)+"delete.png" );
  
  deleteSubalbum = new QPushButton( this );
  deleteSubalbum->setPixmap( *deleteSubalbumImage );
  deleteSubalbum->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );  
  deleteSubalbum->setEnabled(false);
  connect( deleteSubalbum, SIGNAL(clicked()), SLOT(deleteAction()) );

  //place label, listbox, and buttons in grid
  grid = new QGridLayout( this, 3, 2, 0 );
  grid->addMultiCellWidget( subalbumsText, 0, 0, 0, 1 );
  grid->addMultiCellWidget( subalbums, 1, 1, 0, 1 );
  grid->addWidget( createSubalbum, 2, 0);
  grid->addWidget( deleteSubalbum, 2, 1);

  //set listbox to grow
  grid->setRowStretch( 1, 1 );

  //set the background of the widget to be white
  setPaletteBackgroundColor( QColor(255, 255, 255) );
  
  //nothign selected at first
  cachedSelection = NULL;
}
//==============================================
SubalbumsWidget::~SubalbumsWidget()
{
  delete createSubalbumImage;
  delete deleteSubalbumImage;
}
//==============================================
void SubalbumsWidget::createAction()
{
  Subalbum* n = new Subalbum(layout->getWindow()->getTitle()->getAlbum(),
                                             layout->getWindow()->getTitle()->getAlbum()->getNumSubalbums()+1);
  SubalbumPreviewWidget* p = new SubalbumPreviewWidget( subalbums, n );
  p->setDropEnabled(true);
   
  layout->getWindow()->getTitle()->getAlbum()->appendSubalbum( n );
}
//==============================================
void SubalbumsWidget::deleteAction()
{
  //if an item is selected it remove it
  if(subalbums->currentItem() != NULL)
  { 
    //check if user is sure
    QuestionDialog sure( "Delete subalbum?",
                         "Once deleted a subalbum and it's contents cannot be brought back unless a saved copy of the album exists.",
                         "warning.png",
                         this );
    //if user say yes then delete subalbum and refresh
    if(sure.exec())  
    {
      //delete the subalbum
      Subalbum* s = ((SubalbumPreviewWidget*) subalbums->currentItem())->getSubalbum();
      delete subalbums->currentItem();

      //remove subalbum from backend album object
      layout->getWindow()->getTitle()->getAlbum()->removeSubalbum( s );
      
      //rearrange the items in the grid
      subalbums->arrangeItemsInGrid();
      
      //update which subalbum is shown in the layout area
      updateSubalbumLayout(subalbums->firstItem(), false);
      
      //check if any subalbums selected, if not deactivate delete button
      if(subalbums->currentItem() == NULL || !subalbums->currentItem()->isSelected())
      {
          deleteSubalbum->setEnabled(false);
      }
      
      //no images themselves selected so disable set button
      layout->getWindow()->getTitle()->setSetButtonState(false);
    }
  }
}
//==============================================
void SubalbumsWidget::updateSubalbumLayout( QIconViewItem* selection)
{
  //if application busy ignore action
  if(layout->getWindow()->getTitle()->getBusy())
  {
    if(cachedSelection == NULL)
      subalbums->clearSelection();
    else
      subalbums->setSelected(cachedSelection, true);
    return;
  }
  
  //cache selection
  cachedSelection = selection;
  
  //activate/disable delete button depending on if something is currently selected or not
  if(selection == NULL)
    deleteSubalbum->setEnabled(false);
  else
    deleteSubalbum->setEnabled(true);

  updateSubalbumLayout(selection, true);
}
//==============================================
void SubalbumsWidget::updateSubalbumLayout( QIconViewItem* selection, bool oldExists)
{
  //hide photo edit dialog in case we were just editing a photo
  layout->stopEdit(oldExists);

  if(selection == NULL)
    layout->updateSubalbum( NULL, oldExists );
  else
  {
    layout->updateSubalbum(((SubalbumPreviewWidget*)selection)->getSubalbum(), oldExists);
  }
}
//==============================================
void SubalbumsWidget::updateSubalbumName(const QString& val)
{
  //if an item is selected
  if(subalbums->currentItem() != NULL)
  { 
    subalbums->currentItem()->setText(val);
  }
  subalbums->arrangeItemsInGrid();
}
//==============================================
void SubalbumsWidget::updateSubalbumThumbnail( QPixmap* val)
{
  //if an item is selected
  if(subalbums->currentItem() != NULL)
  { 
    subalbums->currentItem()->setPixmap( *val );
  }
  subalbums->arrangeItemsInGrid();
}
//==============================================
void SubalbumsWidget::reorder()
{
  //so item has been moved, reorder linked list of items as necessary
  subalbums->sort( true );
  subalbums->arrangeItemsInGrid();
  
  //get handle on backend album object
  Album* albm = layout->getWindow()->getTitle()->getAlbum();
  
  //sync lists
  albm->syncSubalbumList((SubalbumPreviewWidget*)subalbums->firstItem());
}
//==============================================
void SubalbumsWidget::refresh()
{
  //delete all previous entries
  QIconViewItem* current = subalbums->firstItem();
  QIconViewItem* temp;
  while(current != NULL)
  {
    //get next pointer  
    temp = current->nextItem();
    
    //delete widget
    delete current;
    current = temp;
  }

  //insert all new subalums
  Subalbum* curSubalbum = layout->getWindow()->getTitle()->getAlbum()->getFirstSubalbum();
  while( curSubalbum != NULL)
  {
    (void) new SubalbumPreviewWidget( subalbums, curSubalbum );
    curSubalbum = curSubalbum->getNext();
  }
  
  //refresh layout
  subalbums->arrangeItemsInGrid();
  updateSubalbumLayout( subalbums->firstItem(), false );
  
  //auto select first item
  if(subalbums->firstItem() != NULL)
    subalbums->setSelected( subalbums->firstItem(), TRUE);
}
//==============================================
LayoutWidget* SubalbumsWidget::getParent()
{
  return layout;
}
//==============================================
void SubalbumsWidget::updateButtons(bool enable)
{
  if(!enable)
  {
    buttonsState = createSubalbum->isEnabled();
    createSubalbum->setEnabled(enable);
    deleteSubalbum->setEnabled(enable);  
  }
  else
  {
    createSubalbum->setEnabled(enable);
    deleteSubalbum->setEnabled(buttonsState);  
  }
}
//==============================================
